# -*- coding: utf-8 -*-

import os
from UnicodeHandler import fsencode, fsdecode
import fileinput

class ConfigHandler:
    def __init__(self):
        self._confdir = os.path.join(fsencode(os.getenv('HOME')), '.config','un-applet')
        self._conffile = os.path.join(self._confdir, 'un-applet.conf')
        if not os.path.isdir(self._confdir):
            try:
                os.mkdir(self._confdir)
            except (IOError, OSError):
                print('Error: unable to create directory: "%s".' % self._confdir)

        self._opts = {'TERM_WIDTH': 80,
                      'TERM_HEIGHT': 24,
                      'TERM_XOFF': 100,
                      'TERM_YOFF': 100,
                      'TERM_FG': 'white',
                      'TERM_BG': 'darkblue',
                      'GKSU_CMD': 'gksu',
                      'POLL_INTERVAL': 10,
                      'PREF_TERM': 'mlterm',
                      'STARTUP_DELAY': 2,
                      'ICON_THEME': 'Default'
                      }
        self._int_opts = ('TERM_WIDTH', 'TERM_HEIGHT', 'TERM_XOFF',
                          'TERM_YOFF', 'POLL_INTERVAL', 'STARTUP_DELAY')
        if not os.path.isfile(self._conffile):
            self.save()
        self._load()

    def _get_key_and_value(self, line):
        try:
            l = line.split('=')
            key = l[0].strip()
            value = l[1].strip()
            return (key, value)
        except:
            return ('', '')

    def _load(self):
        try:
            for line in fileinput.input(self._conffile):
                line = line.strip()
                key, value = self._get_key_and_value(line)
                if self._opts.has_key(key):
                    if key in self._int_opts:
                        try:
                            value = int(value)
                        except ValueError:
                            pass
                    self.set(key, value)
        except (IOError, OSError):
            print('Unable to open configuration file "%s".' % self._conffile)

    def get(self, opt=None):
        if opt:
            return self._opts[opt]
        else:
            # query the state of all options
            opts = self._opts.copy()
            return opts

    def set(self, opt, value):
        ok = True
        if opt in self._int_opts:
            try:
                value = int(value)
            except ValueError:
                ok = False
        else:
            value = str(fsencode(value))
        if ok:
            self._opts[opt] = value
        return ok

    def save(self):
        try:
            f = open(self._conffile, 'w')
            for key in self._opts:
                value = str(fsencode(self._opts[key]))
                f.write('%s=%s\n' %(key, value))
            f.close()
        except (IOError, OSError):
            print('Unable to write configuration file "%s".' % self._conffile)

###############################################################################

_handler = None
def initialize():
    global _handler, set, get, save
    if _handler is None:
        _handler = ConfigHandler()
        set = _handler.set
        get = _handler.get
        save = _handler.save
