#!/usr/bin/python
# -*- coding: utf-8 -*-
###############################################################################
# update-notification-applet
# copyright (c) 2015 Michael Lange <klappnase@users.sourceforge.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the #hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################

import sys
import os
import gettext

INSTALLDIR = os.path.abspath(sys.path[0])

# install gettext first
if INSTALLDIR.startswith('/usr/local'):
    localepath = '/usr/local/share/locale'
elif INSTALLDIR.startswith('/usr'):
    localepath = '/usr/share/locale'
else:
    localepath = os.path.join(INSTALLDIR, 'locale')
gettext.install('un-applet', localepath, unicode=1)

import Tkinter
import getopt
import time
import signal

Tkinter.wantobjects = 0

import ConfigHandler
ConfigHandler.initialize()
import updatenotifierapplet

APP_REGISTRY = {
'updatenotifier': updatenotifierapplet.UpdateNotifierApplet
}

def usage():
    print 'un-applet (c) 2015 Michael Lange <klappnase@users.sf.net>\n' +\
          'Usage: un-applet                 : start the update notifier applet\n' +\
          '       un-applet -h , --help     : print this message and exit'

def get_args():
    try:
        opts, files = getopt.getopt(sys.argv[1:], 'h', ['help'])
    except getopt.GetoptError:
        usage()
        sys.exit(1)
    applets = []
    if not opts:
        applets.append('updatenotifier')
    for opt, val in opts:
        if opt in ('-h', '--help'):
            usage()
            sys.exit(0)
    return applets


#############################################################################################
##                                                                                         ##
#############################################################################################

applets = get_args()

# timeout for IceWm system tray
startup_delay = ConfigHandler.get('STARTUP_DELAY')
if startup_delay:
    time.sleep(startup_delay)

class AppletContainer(Tkinter.Tk):
    def __init__(self, *args, **kw):
        Tkinter.Tk.__init__(self, *args, **kw)
        self.apps = {}
        self._id = None
        self.poll_signal()

    def add(self, what):
        self.apps[what] = APP_REGISTRY[what](self)

    def remove(self, what):
        for app in self.apps.keys():
            if self.apps[app] == what:
                self.apps.pop(app)
        if not self.apps:
            self.quit()

    def close_all(self, signum=None, stackframe=None):
        if not self._id is None:
            _id = self._id
            self._id = None
            self.after_cancel(_id)
        if signum:
            print 'un-applet: Received signal %d' % signum
        for app in self.apps.keys():
            self.apps[app].close()

    def poll_signal(self):
        # if a signal is sent to the application, Tk seems to receive
        # this signal only after the mouse enters the window or the
        # window receives keyboard focus or an after() event is triggered,
        # so this "dummy" loop is necessary, so we will not have to wait
        # until something like this happens, especially when only the
        # update-notifier applet is running, which might need quite some
        # time before the next after() loop cycle occurs.
        # see:
        # https://mail.python.org/pipermail/tkinter-discuss/2011-January/002737.html
        # and:
        # https://mail.python.org/pipermail/tkinter-discuss/2011-January/002738.html
        # for a discussion on this
        self._id = self.after(500, self.poll_signal)

def main():
    root = AppletContainer(className='unApplet')
    if Tkinter.TkVersion < 8.6:
        try:
            tkimgver = root.tk.call('package', 'require', 'img::png')
        except Tkinter.TclError:
            print 'un-applet: no png image support available, aborting.'
            print '           Please install the tk image library with "apt-get install libtk-img".'
            root.quit()
            sys.exit()
    root.withdraw()
    def on_return(event):
        event.widget.event_generate('<space>')
    root.bind_class('TButton', '<Return>', on_return)

    for applet in applets:
        root.add(applet)

    root.protocol('WM_DELETE_WINDOW', root.close_all)
    for signum in (2, 3, 15):
        signal.signal(signum, root.close_all)

    root.update_idletasks()
    root.mainloop()
    root.destroy()

if __name__ == '__main__':
    main()
